@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2024 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -189,6 +189,19 @@ void pdf_drop_document(fz_context *ctx, pdf_document *doc);
 
 pdf_document *pdf_keep_document(fz_context *ctx, pdf_document *doc);
 
+/*
+	Do a pass through the document to check if it needs
+	any repairs; and trigger a repair if necessary.
+
+	This is a very expensive operation both in terms of memory use
+	and computation, because it needs to parse the entire file to
+	detect any errors.
+
+	The result of the check is saved, so calling this function again
+	after the initial check is a no-op.
+*/
+void pdf_check_document(fz_context *ctx, pdf_document *doc);
+
 /*
 	down-cast a fz_document to a pdf_document.
 	Returns NULL if underlying document is not PDF
@@ -205,7 +218,7 @@ pdf_page *pdf_page_from_fz_page(fz_context *ctx, fz_page *ptr);
 /*
 	Get a pdf_document handle from an fz_document handle.
 
-	This is superfically similar to pdf_document_from_fz_document
+	This is superficially similar to pdf_document_from_fz_document
 	(and the older pdf_specifics).
 
 	For fz_documents that are actually pdf_documents, this will return
@@ -281,6 +294,27 @@ typedef struct
 */
 void pdf_layer_config_info(fz_context *ctx, pdf_document *doc, int config_num, pdf_layer_config *info);
 
+/*
+	Fetch the creator of the given layer config, or NULL if none exists.
+
+	doc: The document in question.
+
+	config_num: A value in the 0..n-1 range, where n is the
+	value returned from pdf_count_layer_configs.
+*/
+const char *pdf_layer_config_creator(fz_context *ctx, pdf_document *doc, int config_num);
+
+/*
+	Fetch the name of the given layer config, or NULL if none exists.
+
+	doc: The document in question.
+
+	config_num: A value in the 0..n-1 range, where n is the
+	value returned from pdf_count_layer_configs.
+
+*/
+const char *pdf_layer_config_name(fz_context *ctx, pdf_document *doc, int config_num);
+
 /*
 	Set the current configuration.
 	This updates the visibility of the optional content groups
@@ -344,6 +378,9 @@ typedef enum
 	PDF_LAYER_UI_RADIOBOX = 2
 } pdf_layer_config_ui_type;
 
+const char *pdf_layer_config_ui_type_to_string(pdf_layer_config_ui_type type);
+pdf_layer_config_ui_type pdf_layer_config_ui_type_from_string(const char *str);
+
 typedef struct
 {
 	const char *text;
@@ -423,7 +460,9 @@ struct pdf_document
 	fz_stream *file;
 
 	int version;
+	int checked; /* we've checked that we don't need to repair */
 	int is_fdf;
+	int bias;
 	int64_t startxref;
 	int64_t file_size;
 	pdf_crypt *crypt;
@@ -451,7 +490,7 @@ struct pdf_document
 	int map_page_count;
 	pdf_rev_page_map *rev_page_map;
 	pdf_obj **fwd_page_map;
-	int page_tree_broken;
+	int use_page_tree_map;
 
 	int repair_attempted;
 	int repair_in_progress;
@@ -494,8 +533,6 @@ struct pdf_document
 	int hint_obj_offsets_max;
 	int64_t *hint_obj_offsets;
 
-	int resources_localised;
-
 	pdf_lexbuf_large lexbuf;
 
 	pdf_js *js;
@@ -514,6 +551,7 @@ struct pdf_document
 
 	struct {
 		fz_hash_table *fonts;
+		fz_hash_table *colorspaces;
 	} resources;
 
 	int orphans_max;
@@ -576,7 +614,7 @@ pdf_obj *pdf_graft_mapped_object(fz_context *ctx, pdf_graft_map *map, pdf_obj *o
 	destination document of the graft. This involves a deep copy
 	of the objects in question.
 
-	map: A map targetted at the document into which the page should
+	map: A map targeted at the document into which the page should
 	be inserted.
 
 	page_to: The position within the destination document at which
@@ -720,7 +758,7 @@ typedef struct
 	int do_incremental; /* Write just the changed objects. */
 	int do_pretty; /* Pretty-print dictionaries and arrays. */
 	int do_ascii; /* ASCII hex encode binary streams. */
-	int do_compress; /* Compress streams. */
+	int do_compress; /* Compress streams. 1 zlib, 2 brotli */
 	int do_compress_images; /* Compress (or leave compressed) image streams. */
 	int do_compress_fonts; /* Compress (or leave compressed) font streams. */
 	int do_decompress; /* Decompress streams (except when compressing images/fonts). */
@@ -738,6 +776,7 @@ typedef struct
 	int do_preserve_metadata; /* When cleaning, preserve metadata unchanged. */
 	int do_use_objstms; /* Use objstms if possible */
 	int compression_effort; /* 0 for default. 100 = max, 1 = min. */
+	int do_labels; /* Add labels to each object showing how it can be reached from the Root. */
 } pdf_write_options;
 
 FZ_DATA extern const pdf_write_options pdf_default_write_options;
@@ -870,4 +909,32 @@ int pdf_count_page_associated_files(fz_context *ctx, pdf_page *page);
 */
 pdf_obj *pdf_page_associated_file(fz_context *ctx, pdf_page *page, int idx);
 
+
+/*
+	A structure used to create "labels" for numbered objects.
+	The labels are different ways to reach an object from the trailer
+	and page tree, using the "mutool show" syntax.
+
+	Note: Paths involving "Parent", "P", "Prev", and "Last" are ignored,
+	as these are used for cycles in the structures which we don't care about
+	labeling.
+*/
+typedef struct pdf_object_labels pdf_object_labels;
+
+/*
+	Scan the entire object structure to create a directed graph
+	of indirect numbered objects and how they can reach each other.
+*/
+pdf_object_labels *pdf_load_object_labels(fz_context *ctx, pdf_document *doc);
+
+void pdf_drop_object_labels(fz_context *ctx, pdf_object_labels *g);
+
+/*
+	Enumerate all the possible labels for a given numbered object.
+	The callback is invoked with a path for each possible way the object
+	can be reached from the PDF trailer.
+*/
+typedef void (pdf_label_object_fn)(fz_context *ctx, void *arg, const char *label);
+void pdf_label_object(fz_context *ctx, pdf_object_labels *g, int num, pdf_label_object_fn *callback, void *arg);
+
 #endif
